@@ -28,5 +28,7 @@ def upgrade():
 
 
 def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_contacts_authenticate_qualify_yesno_values','ps_contacts')
     op.drop_column('ps_contacts', 'authenticate_qualify')
 
