@@ -847,6 +847,14 @@ ALTER TABLE ps_transports ALTER COLUMN tos VARCHAR(10);
 
 GO
 
+declare @const_name varchar(256)
+select @const_name = [name] from sys.check_constraints
+where parent_object_id = object_id('ps_transports')
+and col_name(parent_object_id, parent_column_id) = 'cos'
+exec('alter table ps_transports drop constraint ' + @const_name);
+
+GO
+
 ALTER TABLE ps_transports DROP COLUMN cos;
 
 GO
@@ -1007,6 +1015,10 @@ GO
 
 -- Running upgrade d39508cb8d8 -> 5950038a6ead
 
+ALTER TABLE ps_transports DROP CONSTRAINT yesno_values;
+
+GO
+
 ALTER TABLE ps_transports ALTER COLUMN verifiy_server VARCHAR(3);
 
 GO
@@ -1015,7 +1027,11 @@ EXEC sp_rename 'ps_transports.verifiy_server', verify_server, 'COLUMN';
 
 GO
 
-ALTER TABLE ps_transports ADD CONSTRAINT yesno_values CHECK (verifiy_server IN ('yes', 'no'));
+ALTER TABLE ps_transports ALTER COLUMN verify_server VARCHAR(3);
+
+GO
+
+ALTER TABLE ps_transports ADD CONSTRAINT yesno_values CHECK (verify_server IN ('yes', 'no'));
 
 GO
 
@@ -1501,6 +1517,10 @@ GO
 
 -- Running upgrade c7a44a5a0851 -> 3772f8f828da
 
+ALTER TABLE ps_endpoints DROP CONSTRAINT ck_ps_endpoints_identify_by_pjsip_identify_by_values;
+
+GO
+
 ALTER TABLE ps_endpoints ALTER COLUMN identify_by VARCHAR(13);
 
 GO
@@ -1565,6 +1585,232 @@ UPDATE alembic_version SET version_num='28ab27a7826d' WHERE alembic_version.vers
 
 GO
 
+-- Running upgrade 28ab27a7826d -> 465e70e8c337
+
+ALTER TABLE ps_endpoint_id_ips ADD match_header VARCHAR(255) NULL;
+
+GO
+
+UPDATE alembic_version SET version_num='465e70e8c337' WHERE alembic_version.version_num = '28ab27a7826d';
+
+GO
+
+-- Running upgrade 465e70e8c337 -> 15db7b91a97a
+
+ALTER TABLE ps_endpoints ADD rtcp_mux VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (rtcp_mux IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='15db7b91a97a' WHERE alembic_version.version_num = '465e70e8c337';
+
+GO
+
+-- Running upgrade 15db7b91a97a -> f638dbe2eb23
+
+ALTER TABLE ps_transports ADD symmetric_transport VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_transports ADD CONSTRAINT yesno_values CHECK (symmetric_transport IN ('yes', 'no'));
+
+GO
+
+ALTER TABLE ps_subscription_persistence ADD contact_uri VARCHAR(256) NULL;
+
+GO
+
+UPDATE alembic_version SET version_num='f638dbe2eb23' WHERE alembic_version.version_num = '15db7b91a97a';
+
+GO
+
+-- Running upgrade f638dbe2eb23 -> 8fce4c573e15
+
+ALTER TABLE ps_endpoints ADD allow_overlap VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (allow_overlap IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='8fce4c573e15' WHERE alembic_version.version_num = 'f638dbe2eb23';
+
+GO
+
+-- Running upgrade 8fce4c573e15 -> 2da192dbbc65
+
+CREATE TABLE ps_outbound_publishes (
+    id VARCHAR(40) NOT NULL, 
+    expiration INTEGER NULL, 
+    outbound_auth VARCHAR(40) NULL, 
+    outbound_proxy VARCHAR(256) NULL, 
+    server_uri VARCHAR(256) NULL, 
+    from_uri VARCHAR(256) NULL, 
+    to_uri VARCHAR(256) NULL, 
+    event VARCHAR(40) NULL, 
+    max_auth_attempts INTEGER NULL, 
+    transport VARCHAR(40) NULL, 
+    multi_user VARCHAR(3) NULL, 
+    [@body] VARCHAR(40) NULL, 
+    [@context] VARCHAR(256) NULL, 
+    [@exten] VARCHAR(256) NULL, 
+    UNIQUE (id), 
+    CONSTRAINT yesno_values CHECK (multi_user IN ('yes', 'no'))
+);
+
+GO
+
+CREATE INDEX ps_outbound_publishes_id ON ps_outbound_publishes (id);
+
+GO
+
+CREATE TABLE ps_inbound_publications (
+    id VARCHAR(40) NOT NULL, 
+    endpoint VARCHAR(40) NULL, 
+    [event_asterisk-devicestate] VARCHAR(40) NULL, 
+    [event_asterisk-mwi] VARCHAR(40) NULL, 
+    UNIQUE (id)
+);
+
+GO
+
+CREATE INDEX ps_inbound_publications_id ON ps_inbound_publications (id);
+
+GO
+
+CREATE TABLE ps_asterisk_publications (
+    id VARCHAR(40) NOT NULL, 
+    devicestate_publish VARCHAR(40) NULL, 
+    mailboxstate_publish VARCHAR(40) NULL, 
+    device_state VARCHAR(3) NULL, 
+    device_state_filter VARCHAR(256) NULL, 
+    mailbox_state VARCHAR(3) NULL, 
+    mailbox_state_filter VARCHAR(256) NULL, 
+    UNIQUE (id), 
+    CONSTRAINT yesno_values CHECK (device_state IN ('yes', 'no')), 
+    CONSTRAINT yesno_values CHECK (mailbox_state IN ('yes', 'no'))
+);
+
+GO
+
+CREATE INDEX ps_asterisk_publications_id ON ps_asterisk_publications (id);
+
+GO
+
+UPDATE alembic_version SET version_num='2da192dbbc65' WHERE alembic_version.version_num = '8fce4c573e15';
+
+GO
+
+-- Running upgrade 2da192dbbc65 -> 1d0e332c32af
+
+CREATE TABLE ps_resource_list (
+    id VARCHAR(40) NOT NULL, 
+    list_item VARCHAR(2048) NULL, 
+    event VARCHAR(40) NULL, 
+    full_state VARCHAR(3) NULL, 
+    notification_batch_interval INTEGER NULL, 
+    UNIQUE (id), 
+    CONSTRAINT yesno_values CHECK (full_state IN ('yes', 'no'))
+);
+
+GO
+
+CREATE INDEX ps_resource_list_id ON ps_resource_list (id);
+
+GO
+
+UPDATE alembic_version SET version_num='1d0e332c32af' WHERE alembic_version.version_num = '2da192dbbc65';
+
+GO
+
+-- Running upgrade 1d0e332c32af -> 86bb1efa278d
+
+ALTER TABLE ps_endpoints ADD refer_blind_progress VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (refer_blind_progress IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='86bb1efa278d' WHERE alembic_version.version_num = '1d0e332c32af';
+
+GO
+
+-- Running upgrade 86bb1efa278d -> d7983954dd96
+
+ALTER TABLE ps_endpoints ADD notify_early_inuse_ringing VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (notify_early_inuse_ringing IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='d7983954dd96' WHERE alembic_version.version_num = '86bb1efa278d';
+
+GO
+
+-- Running upgrade d7983954dd96 -> 164abbd708c
+
+ALTER TABLE ps_endpoints DROP CONSTRAINT pjsip_dtmf_mode_values_v2;
+
+GO
+
+ALTER TABLE ps_endpoints ALTER COLUMN dtmf_mode VARCHAR(9);
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT pjsip_dtmf_mode_values_v3 CHECK (dtmf_mode IN ('rfc4733', 'inband', 'info', 'auto', 'auto_info'));
+
+GO
+
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96';
+
+GO
+
+-- Running upgrade 164abbd708c -> f3d1c5d38b56
+
+ALTER TABLE ps_contacts ADD prune_on_boot VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_contacts ADD CONSTRAINT yesno_values CHECK (prune_on_boot IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c';
+
+GO
+
+-- Running upgrade f3d1c5d38b56 -> b83645976fdd
+
+ALTER TABLE ps_endpoints ADD dtls_fingerprint VARCHAR(7) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT sha_hash_values CHECK (dtls_fingerprint IN ('SHA-1', 'SHA-256'));
+
+GO
+
+UPDATE alembic_version SET version_num='b83645976fdd' WHERE alembic_version.version_num = 'f3d1c5d38b56';
+
+GO
+
+-- Running upgrade b83645976fdd -> a1698e8bb9c5
+
+ALTER TABLE ps_endpoints ADD incoming_mwi_mailbox VARCHAR(40) NULL;
+
+GO
+
+UPDATE alembic_version SET version_num='a1698e8bb9c5' WHERE alembic_version.version_num = 'b83645976fdd';
+
+GO
+
 COMMIT;
 
 GO
