@@ -341,10 +341,7 @@ static void calendar_destructor(void *obj)
 	}
 	ast_calendar_clear_events(cal);
 	ast_string_field_free_memory(cal);
-	if (cal->vars) {
-		ast_variables_destroy(cal->vars);
-		cal->vars = NULL;
-	}
+	ast_variables_destroy(cal->vars);
 	ao2_ref(cal->events, -1);
 	ao2_unlock(cal);
 }
@@ -406,28 +403,22 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 {
 	struct ast_calendar *cal;
 	struct ast_variable *v, *last = NULL;
-	int new_calendar = 0;
 
-	if (!(cal = find_calendar(cat))) {
-		new_calendar = 1;
-		if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
-			ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
-			return NULL;
-		}
+	if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
+		ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
+		return NULL;
+	}
 
-		if (!(cal->events = ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn))) {
-			ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
-			cal = unref_calendar(cal);
-			return NULL;
-		}
+	if (!(cal->events = ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn))) {
+		ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
+		cal = unref_calendar(cal);
+		return NULL;
+	}
 
-		if (ast_string_field_init(cal, 32)) {
-			ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
-			cal = unref_calendar(cal);
-			return NULL;
-		}
-	} else {
-		cal->pending_deletion = 0;
+	if (ast_string_field_init(cal, 32)) {
+		ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
+		cal = unref_calendar(cal);
+		return NULL;
 	}
 
 	ast_string_field_set(cal, name, cat);
@@ -482,17 +473,22 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 		}
 	}
 
-	if (new_calendar) {
-		cal->thread = AST_PTHREADT_NULL;
-		ast_cond_init(&cal->unload, NULL);
-		ao2_link(calendars, cal);
-		if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
-			/* If we start failing to create threads, go ahead and return NULL
-			 * and the tech module will be unregistered
-			 */ 
-			ao2_unlink(calendars, cal);
-			cal = unref_calendar(cal);
-		}
+	if (cal->autoreminder && ast_strlen_zero(cal->notify_channel)) {
+		ast_log(LOG_WARNING,
+				"You have set 'autoreminder' but not 'channel' for calendar '%s.' "
+				"Notifications will not occur.\n",
+				cal->name);
+	}
+
+	cal->thread = AST_PTHREADT_NULL;
+	ast_cond_init(&cal->unload, NULL);
+	ao2_link(calendars, cal);
+	if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
+		/* If we start failing to create threads, go ahead and return NULL
+		 * and the tech module will be unregistered
+		 */
+		ao2_unlink(calendars, cal);
+		cal = unref_calendar(cal);
 	}
 
 	return cal;
@@ -735,7 +731,7 @@ static void *do_notify(void *data)
 	struct ast_channel *chan = NULL;
 	struct ast_variable *itervar;
 	char *tech, *dest;
-	char buf[8];
+	char buf[33];
 	struct ast_format_cap *caps;
 
 	tech = ast_strdupa(event->owner->notify_channel);
@@ -948,7 +944,7 @@ static int schedule_calendar_event(struct ast_calendar *cal, struct ast_calendar
 	event = cmp_event ? cmp_event : old_event;
 
 	ao2_lock(event);
-	if (!cmp_event || old_event->alarm != event->alarm) {
+	if (!ast_strlen_zero(cal->notify_channel) && (!cmp_event || old_event->alarm != event->alarm)) {
 		changed = 1;
 		if (cal->autoreminder) {
 			alarm_notify_sched = (event->start - (60 * cal->autoreminder) - now.tv_sec) * 1000;
@@ -957,7 +953,7 @@ static int schedule_calendar_event(struct ast_calendar *cal, struct ast_calendar
 		}
 
 		/* For now, send the notification if we missed it, but the meeting hasn't happened yet */
-		if (event->start >=  now.tv_sec) {
+		if (event->start >= now.tv_sec) {
 			if (alarm_notify_sched <= 0) {
 				alarm_notify_sched = 1;
 			}
@@ -1590,7 +1586,7 @@ static char *epoch_to_string(char *buf, size_t buflen, time_t epoch)
 
 static char *handle_show_calendar(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-#define FORMAT "%-17.17s : %-20.20s\n"
+#define FORMAT  "%-18.18s : %-20.20s\n"
 #define FORMAT2 "%-12.12s: %-40.60s\n"
 	struct ao2_iterator i;
 	struct ast_calendar *cal;
@@ -1639,7 +1635,13 @@ static char *handle_show_calendar(struct ast_cli_entry *e, int cmd, struct ast_c
 	ast_cli(a->fd, FORMAT, "Notify appdata", cal->notify_appdata);
 	ast_cli(a->fd, "%-17.17s : %d\n", "Refresh time", cal->refresh);
 	ast_cli(a->fd, "%-17.17s : %d\n", "Timeframe", cal->timeframe);
-	ast_cli(a->fd, "%-17.17s : %d\n", "Autoreminder", cal->autoreminder);
+
+	if (cal->autoreminder) {
+		ast_cli(a->fd, "%-17.17s : %d minutes before event\n", "Autoreminder", cal->autoreminder);
+	} else {
+		ast_cli(a->fd, "%-17.17s : None\n", "Autoreminder");
+	}
+
 	ast_cli(a->fd, "%s\n", "Events");
 	ast_cli(a->fd, "%s\n", "------");
 
@@ -1757,30 +1759,16 @@ static struct ast_custom_function calendar_event_function = {
 	.read = calendar_event_read,
 };
 
-static int cb_pending_deletion(void *user_data, void *arg, int flags)
-{
-	struct ast_calendar *cal = user_data;
-
-	cal->pending_deletion = 1;
-
-	return CMP_MATCH;
-}
-
-static int cb_rm_pending_deletion(void *user_data, void *arg, int flags)
-{
-	struct ast_calendar *cal = user_data;
-
-	return cal->pending_deletion ? CMP_MATCH : 0;
-}
-
 static int reload(void)
 {
 	struct ast_calendar_tech *iter;
 
 	ast_mutex_lock(&reloadlock);
 
-	/* Mark existing calendars for deletion */
-	ao2_callback(calendars, OBJ_NODATA | OBJ_MULTIPLE, cb_pending_deletion, NULL);
+	/* Delete all of the calendars */
+	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
+
+	/* Load configuration */
 	load_config(1);
 
 	AST_LIST_LOCK(&techs);
@@ -1791,9 +1779,6 @@ static int reload(void)
 	}
 	AST_LIST_UNLOCK(&techs);
 
-	/* Delete calendars that no longer show up in the config */
-	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, cb_rm_pending_deletion, NULL);
-
 	ast_mutex_unlock(&reloadlock);
 
 	return 0;
@@ -1882,7 +1867,7 @@ static int load_module(void)
 {
 	if (!(calendars = ao2_container_alloc(CALENDAR_BUCKETS, calendar_hash_fn, calendar_cmp_fn))) {
 		ast_log(LOG_ERROR, "Unable to allocate calendars container!\n");
-		return AST_MODULE_LOAD_FAILURE;
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (load_config(0)) {
@@ -1896,7 +1881,9 @@ static int load_module(void)
 
 	if (!(sched = ast_sched_context_create())) {
 		ast_log(LOG_ERROR, "Unable to create sched context\n");
-		return AST_MODULE_LOAD_FAILURE;
+		ast_config_destroy(calendar_config);
+		calendar_config = NULL;
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (ast_pthread_create_background(&refresh_thread, NULL, do_refresh, NULL) < 0) {
