@@ -133,6 +133,29 @@ int ast_sip_initialize_distributor(void);
  */
 void ast_sip_destroy_distributor(void);
 
+/*!
+ * \internal
+ * \brief Initialize the transport events notify module
+ * \since 13.18.0
+ *
+ * The transport events notify module is responsible for monitoring
+ * when transports die and calling any registered callbacks when that
+ * happens.  It also manages any PJPROJECT transport state callbacks
+ * registered to it so the callbacks be more dynamic allowing module
+ * loading/unloading.
+ *
+ * \retval -1 Failure
+ * \retval 0 Success
+ */
+int ast_sip_initialize_transport_events(void);
+
+/*!
+ * \internal
+ * \brief Destruct the transport events notify module.
+ * \since 13.18.0
+ */
+void ast_sip_destroy_transport_events(void);
+
 /*!
  * \internal
  * \brief Initialize global type on a sorcery instance
@@ -189,7 +212,7 @@ int ast_res_pjsip_init_options_handling(int reload);
  * \retval 0 on success
  * \retval other on failure
  */
-int ast_res_pjsip_init_message_ip_updater(void);
+int ast_res_pjsip_init_message_filter(void);
 
 /*!
  * \internal
@@ -258,7 +281,7 @@ void ast_res_pjsip_cleanup_options_handling(void);
  * \internal
  * \brief Clean up res_pjsip message ip updating handling
  */
-void ast_res_pjsip_cleanup_message_ip_updater(void);
+void ast_res_pjsip_cleanup_message_filter(void);
 
 /*!
  * \internal
@@ -315,6 +338,18 @@ int internal_sip_register_service(pjsip_module *module);
  */
 int internal_sip_unregister_service(pjsip_module *module);
 
+/*!
+ * \internal
+ * \brief Used by res_pjsip.so to register a supplement without adding a self reference
+ */
+void internal_sip_register_supplement(struct ast_sip_supplement *supplement);
+
+/*!
+ * \internal
+ * \brief Used by res_pjsip.so to unregister a supplement without removing a self reference
+ */
+int internal_sip_unregister_supplement(struct ast_sip_supplement *supplement);
+
 /*!
  * \internal
  * \brief Used by res_pjsip.so to register an endpoint formatter without adding a self reference
@@ -327,6 +362,20 @@ void internal_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter
  */
 int internal_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
 
+struct ast_sip_session_supplement;
+
+/*!
+ * \internal
+ * \brief Used by res_pjsip.so to register a session supplement without adding a self reference
+ */
+void internal_sip_session_register_supplement(struct ast_sip_session_supplement *supplement);
+
+/*!
+ * \internal
+ * \brief Used by res_pjsip.so to unregister a session supplement without removing a self reference
+ */
+int internal_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement);
+
 /*!
  * \internal
  * \brief Finds or creates contact_status for a contact
@@ -358,4 +407,16 @@ int ast_sip_initialize_scheduler(void);
  */
 int ast_sip_destroy_scheduler(void);
 
+/*!
+ * \internal
+ * \brief Add a reference to the res_pjsip module
+ */
+void internal_res_pjsip_ref(void);
+
+/*!
+ * \internal
+ * \brief Remove a reference from the res_pjsip module
+ */
+void internal_res_pjsip_unref(void);
+
 #endif /* RES_PJSIP_PRIVATE_H_ */
